@@ -24,6 +24,7 @@ import json
 import netaddr
 import re
 import socket
+import uuid
 import weakref
 
 # OpenStack imports.
@@ -42,37 +43,27 @@ from calico import datamodel_v1
 from calico import etcdutils
 import etcd
 from eventlet.semaphore import Semaphore
+from networking_calico.common import config as calico_config
 from networking_calico.plugins.ml2.drivers.calico.election import Elector
 
 
 # The node hostname is used as the default identity for leader election
 _hostname = socket.gethostname()
 
+
+# Elector configuration;
+elector_opt = cfg.StrOpt(
+    'elector_name', default=_hostname,
+    help="A unique name to identify this node in leader election"
+)
+
+# Register Calico related configuration options
+calico_config.register_options(cfg.CONF, additional_options=[elector_opt])
+
+
 # The amount of time in seconds to wait for etcd responses.
 ETCD_TIMEOUT = 5
 
-# Register Calico-specific options.
-calico_opts = [
-    # etcd connection information.
-    cfg.StrOpt('etcd_host', default='127.0.0.1',
-               help="The hostname or IP of the etcd node/proxy"),
-    cfg.IntOpt('etcd_port', default=4001,
-               help="The port to use for the etcd node/proxy"),
-    # etcd TLS-related options.
-    cfg.StrOpt('etcd_key_file', default=None,
-               help="The path to the TLS key file to use with etcd."),
-    cfg.StrOpt('etcd_cert_file', default=None,
-               help="The path to the TLS client certificate file to use with "
-                    "etcd."),
-    cfg.StrOpt('etcd_ca_cert_file', default=None,
-               help="The path to the TLS CA certificate file to use with "
-                    "etcd."),
-    # Elector configuration.
-    cfg.StrOpt('elector_name', default=_hostname,
-               help="A unique name to identify this node in leader election"),
-]
-cfg.CONF.register_opts(calico_opts, 'calico')
-
 OPENSTACK_ENDPOINT_RE = re.compile(
     r'^' + datamodel_v1.HOST_DIR +
     r'/(?P<hostname>[^/]+)/.*openstack.*/endpoint/(?P<endpoint_id>[^/]+)')
@@ -343,8 +334,29 @@ class CalicoTransportEtcd(object):
         """
         LOG.info("Providing Felix configuration")
 
-        # First read the config values, so as to avoid unnecessary
-        # writes.
+        # First create the ClusterGUID, if it is not already set.
+        cluster_guid_key = datamodel_v1.key_for_config('ClusterGUID')
+        try:
+            cluster_guid = self.client.read(cluster_guid_key).value
+            LOG.info('ClusterGUID is %s', cluster_guid)
+        except etcd.EtcdKeyNotFound:
+            # Generate and write a globally unique cluster GUID.  Write it
+            # idempotently into the datastore. The prevExist=False creates the
+            # value (safely with CaS) if it doesn't exist.
+            LOG.info('ClusterGUID not set yet (%s)', cluster_guid_key)
+            guid = uuid.uuid4()
+            guid_string = guid.get_hex()
+            try:
+                self.client.write(cluster_guid_key,
+                                  guid_string,
+                                  prevExist=False)
+            except etcd.EtcdAlreadyExist:
+                LOG.info('ClusterGUID is now set - another orchestrator or' +
+                         ' Neutron server instance must have just written it')
+                pass
+
+        # Read other config values that should exist.  We will write them only
+        # if they're not already (collectively) set as we want them.
         prefix = None
         reporting_enabled = None
         ready = None
@@ -362,7 +374,7 @@ class CalicoTransportEtcd(object):
             LOG.info('%s -> tap', iface_pfx_key)
             self.client.write(iface_pfx_key, 'tap')
         if reporting_enabled != "true":
-            LOG.info('%s -> true', reporting_enabled)
+            LOG.info('%s -> true', reporting_key)
             self.client.write(reporting_key, 'true')
         if ready != 'true':
             # TODO(nj) Set this flag only once we're really ready!
@@ -1022,7 +1034,9 @@ def _neutron_rule_to_etcd_rule(rule):
 
 
 def subnet_etcd_data(subnet):
-    data = {'cidr': str(netaddr.IPNetwork(subnet['cidr'])),
+    data = {'network_id': subnet['network_id'],
+            'cidr': str(netaddr.IPNetwork(subnet['cidr'])),
+            'host_routes': subnet['host_routes'],
             'gateway_ip': subnet['gateway_ip']}
     if subnet['dns_nameservers']:
         data['dns_servers'] = subnet['dns_nameservers']
