@@ -26,8 +26,11 @@ from calico import datamodel_v1
 from calico.etcdutils import EtcdWatcher
 from networking_calico.agent.dhcp_agent import CalicoDhcpAgent
 from networking_calico.agent.dhcp_agent import FakePlugin
+from networking_calico.agent.dhcp_agent import get_etcd_connection_settings
 from networking_calico.agent.linux.dhcp import DnsmasqRouted
+from networking_calico.common import config as calico_config
 from neutron.agent.dhcp_agent import register_options
+from neutron.agent.linux import dhcp
 from neutron.common import constants
 from neutron.tests import base
 from oslo_config import cfg
@@ -42,12 +45,15 @@ class TestFakePlugin(base.BaseTestCase):
         self.plugin = FakePlugin()
 
     def test_create(self):
-        port = self.plugin.create_dhcp_port({'port': {}})
-        self.assertEqual(port, {
-            'device_owner': 'network:dhcp',
-            'id': 'dhcp',
-            'mac_address': '02:00:00:00:00:00'
+        port = self.plugin.create_dhcp_port({
+            'port': {'network_id': 'net-id-0'}
         })
+        self.assertEqual({
+            'network_id': 'net-id-0',
+            'device_owner': 'network:dhcp',
+            'id': 'net-id-0',
+            'mac_address': '02:00:00:00:00:00'},
+            port)
 
     def test_release(self):
         self.plugin.release_dhcp_port('calico', 'dhcp')
@@ -57,6 +63,7 @@ class TestDhcpAgent(base.BaseTestCase):
     def setUp(self):
         super(TestDhcpAgent, self).setUp()
         register_options(cfg.CONF)
+        calico_config.register_options(cfg.CONF)
 
     @mock.patch('etcd.Client')
     def test_mainline(self, etcd_client_cls):
@@ -74,7 +81,7 @@ class TestDhcpAgent(base.BaseTestCase):
             etcd_snapshot_response = mock.Mock()
             etcd_snapshot_response.leaves = []
             agent.etcd._on_snapshot_loaded(etcd_snapshot_response)
-            call_driver.assert_called_with('reload_allocations', mock.ANY)
+            call_driver.assert_not_called()
 
         # Prepare subnet reads for the endpoints that we will notify.
         self.first_workload_read = True
@@ -82,11 +89,12 @@ class TestDhcpAgent(base.BaseTestCase):
         def etcd_client_read(key, **kwargs):
             LOG.info('etcd_client_read %s %s', key, kwargs)
             if 'v4subnet-1' in key:
-                self.assertEqual(key,
-                                 '/calico/dhcp/v1/subnet/v4subnet-1')
+                self.assertEqual('/calico/dhcp/v1/subnet/v4subnet-1',
+                                 key)
                 return EtcdResponse(value=json.dumps({
                     'cidr': '10.28.0.0/24',
-                    'gateway_ip': '10.28.0.1'
+                    'gateway_ip': '10.28.0.1',
+                    'host_routes': []
                 }))
             if 'v6subnet-1' in key:
                 return EtcdResponse(value=json.dumps({
@@ -96,7 +104,9 @@ class TestDhcpAgent(base.BaseTestCase):
             if 'v4subnet-2' in key:
                 return EtcdResponse(value=json.dumps({
                     'cidr': '10.29.0.0/24',
-                    'gateway_ip': '10.29.0.1'
+                    'gateway_ip': '10.29.0.1',
+                    'host_routes': [{'destination': '11.11.0.0/16',
+                                     'nexthop': '10.65.0.1'}]
                 }))
             if key == '/calico/v1/host/nj-ubuntu/workload':
                 if self.first_workload_read:
@@ -188,8 +198,8 @@ class TestDhcpAgent(base.BaseTestCase):
             # Check no further etcd reads.
             etcd_client.read.assert_not_called()
 
-            # Check DHCP driver was asked to reload allocations.
-            call_driver.assert_called_with('reload_allocations', mock.ANY)
+            # Check DHCP driver was asked to restart.
+            call_driver.assert_called_with('restart', mock.ANY)
 
             # Notify deletion of the first endpoint.
             agent.etcd.on_endpoint_delete(None,
@@ -202,7 +212,7 @@ class TestDhcpAgent(base.BaseTestCase):
             etcd_client.read.assert_not_called()
 
             # Check DHCP driver was asked to reload allocations.
-            call_driver.assert_called_with('reload_allocations', mock.ANY)
+            call_driver.assert_called_with('restart', mock.ANY)
 
             # Notify another endpoint using a new subnet.
             agent.etcd.on_endpoint_set(EtcdResponse(value=json.dumps({
@@ -236,7 +246,7 @@ class TestDhcpAgent(base.BaseTestCase):
             eventlet.spawn(agent.etcd.loop)
 
             # Report that the subnet watcher noticed a change.
-            agent.etcd.subnet_watcher.on_subnet_set(None, 'some-subnet-X')
+            agent.etcd.on_subnet_set(None, 'some-subnet-X')
             eventlet.sleep(0.2)
 
             # Check DHCP driver was asked to restart.
@@ -293,8 +303,9 @@ class TestDhcpAgent(base.BaseTestCase):
             ])
             etcd_client.read.reset_mock()
 
-            # Check DHCP driver was asked to restart.
-            call_driver.assert_called_with('restart', mock.ANY)
+            # Check DHCP driver was not troubled - because the subnet data was
+            # missing and so the port could not be processed further.
+            call_driver.assert_not_called()
 
     @mock.patch('etcd.Client')
     def test_dir_delete(self, etcd_client_cls):
@@ -374,7 +385,7 @@ class TestDhcpAgent(base.BaseTestCase):
 
         # To test handling of SubnetWatcher's loop exiting, make
         # EtcdWatcher.loop throw an exception.
-        loop_fn.side_effect = Exception
+        loop_fn.side_effect = Exception('from test_kill_agent')
 
         # Create the DHCP agent and allow it to start the SubnetWatcher loop.
         agent = CalicoDhcpAgent()
@@ -467,17 +478,49 @@ class TestDhcpAgent(base.BaseTestCase):
             'endpoint-1'
         )
 
-        # Check expected subnets were read from etcd.
-        etcd_client.read.assert_has_calls([
+        # Check that either the v4 or the v6 subnet was read from etcd.  Since
+        # it's invalid, the processing of the new endpoint stops at that point,
+        # and the other subnet is not read at all.
+        read_calls = etcd_client.read.mock_calls
+        self.assertEqual(1, len(read_calls))
+        self.assertTrue(read_calls[0] in [
             mock.call(datamodel_v1.key_for_subnet('v4subnet-1'),
                       consistent=True),
             mock.call(datamodel_v1.key_for_subnet('v6subnet-1'),
-                      consistent=True)
-        ],
-            any_order=True
-        )
+                      consistent=True),
+        ])
         etcd_client.read.reset_mock()
 
+    @mock.patch('etcd.Client')
+    def test_etcd_watchers_init_with_conf_values(self, *_):
+        agent = CalicoDhcpAgent()
+
+        provided_kwargs = get_etcd_connection_settings()
+
+        def check_provided_args(watcher_obj, expected_key_to_poll):
+            for attr_name in ('etcd_scheme', 'etcd_key',
+                              'etcd_cert', 'etcd_ca'):
+                self.assertEqual(
+                    getattr(watcher_obj, attr_name),
+                    provided_kwargs[attr_name]
+                )
+
+            # provided hosts are stored in form of list [(host, port)...]
+            # on etcd watcher
+            etcd_host, etcd_port = provided_kwargs['etcd_addrs'].split(':')
+            self.assertEqual(watcher_obj.etcd_hosts,
+                             [(etcd_host, int(etcd_port))])
+
+            self.assertEqual(watcher_obj.key_to_poll, expected_key_to_poll)
+
+        expected_key = \
+            datamodel_v1.dir_for_host(socket.gethostname()) + '/workload'
+        check_provided_args(agent.etcd, expected_key)
+
+        expected_key = datamodel_v1.SUBNET_DIR
+        check_provided_args(agent.etcd.subnet_watcher, expected_key)
+
+
 commonutils = 'neutron.agent.linux.dhcp.commonutils'
 try:
     from neutron.agent.linux.dhcp import commonutils as xxx  # noqa
@@ -515,11 +558,17 @@ class TestDnsmasqRouted(base.BaseTestCase):
         network.id = 'calico'
         network.subnets = [v4subnet, v6subnet]
         network.mtu = 0
+        network.ports = [
+            dhcp.DictModel({'device_id': 'tap1'}),
+            dhcp.DictModel({'device_id': 'tap2'}),
+            dhcp.DictModel({'device_id': 'tap3'}),
+        ]
         device_mgr_cls.return_value.driver.bridged = False
-        cmdline = DnsmasqRouted(cfg.CONF,
-                                network,
-                                None)._build_cmdline_callback('/run/pid_file')
-        self.assertEqual(cmdline, [
+        dhcp_driver = DnsmasqRouted(cfg.CONF, network, None)
+        with mock.patch.object(dhcp_driver, '_get_value_from_conf_file') as gv:
+            gv.return_value = 'ns-dhcp'
+            cmdline = dhcp_driver._build_cmdline_callback('/run/pid_file')
+        self.assertEqual([
             'dnsmasq',
             '--no-hosts',
             '--no-resolv',
@@ -532,13 +581,15 @@ class TestDnsmasqRouted(base.BaseTestCase):
             '--dhcp-leasefile=/run/calico/leases',
             '--dhcp-match=set:ipxe,175',
             '--bind-dynamic',
-            '--interface=None',
-            '--interface=tap*',
-            '--bridge-interface=None,tap*',
+            '--interface=ns-dhcp',
             '--dhcp-range=set:tag0,10.28.0.0,static,86400s',
             '--dhcp-lease-max=16777216',
             '--conf-file=',
             '--domain=openstacklocal',
             '--dhcp-range=set:tag1,2001:db8:1::,static,off-link,80,86400s',
-            '--enable-ra'
-        ])
+            '--enable-ra',
+            '--interface=tap1',
+            '--interface=tap2',
+            '--interface=tap3',
+            '--bridge-interface=ns-dhcp,tap1,tap2,tap3'],
+            cmdline)
